;*******************************************************************************
;* TITLE	: GPU.S							       *
;* Function	: GPU Interface Routines				       *
;*									       *
;* Project #	: JAGUAR 3D pack out					       *
;*									       *
;* Programmer	: Andrew J Burgess					       *
;*		  Rob Zdybel						       *
;*                                                                             *
;* COPYRIGHT 1992,1993 Atari U.S. Corporation				       *
;* UNATHORIZED REPRODUCTION, ADAPTATION, DISTRIBUTION, PERFORMANCE OR DISPLAY  *
;* OF THIS COMPUTER PROGRAM OR THE ASSOCIATED AUDIOVISUAL WORK IS STRICTLY     *
;* PROHIBITED.  ALL RIGHTS RESERVED.					       *
;*******************************************************************************

	.title	"GPU.S - GPU Interface Routines"

;*=============================================================================*
;* INCLUDE FILES							       *
;*=============================================================================*

	include "jaguar.inc"
	include "blit.inc"
	include "gpu.inc"

;*=============================================================================*
;* EXTERNAL ROUTINES							       *
;*=============================================================================*

;*=============================================================================*
;* EXTERNAL DATA							       *
;*=============================================================================*

	.extern	framecnt

;*=============================================================================*
;* PUBLIC ROUTINES							       *
;*=============================================================================*

	.globl	gpuload
	.globl	gpurun
	.globl	gpuwait

;*=============================================================================*
;* PUBLIC DATA								       *
;*=============================================================================*

;*=============================================================================*
;* LOCAL EQUATES							       *
;*=============================================================================*

;*******************************************************************************
;* SUBROUTINE AREA							       *
;*******************************************************************************

	.text

;*=============================================================================*
;* GPULOAD	- load a GPU program into GPU RAM			       *
;*									       *
;* Given:								       *
;*	a0 = 68000 address of GPU program				       *
;*	a1 = destination address					       *
;*	d0 = size of program						       *
;*	Control								       *
;*									       *
;* Returns:								       *
;*	GPU program loaded in GPU local memory				       *
;*									       *
;* Register Usage:							       *
;*	All Registers Restored						       *
;*									       *
;* Externals:								       *
;*	None								       *
;*									       *
;*=============================================================================*

gpuload:
	movem.l	d0-d1/a0-a2,-(sp)	; save registers

	move.l	#B_CMD,a2
.waitblit:				; Wait for BLTTER Idle
	move.l	(a2),d1
	lsr.w	#1,d1
	bcc	.waitblit

.gpuwait:
	jsr	gpuwait			; Wait for GPU to finish

; This code will load a gpu program into the Blitter at the address
; specified in the header 

	move.l	#(PITCH1|PIXEL16|WID256|XADDPHR),d1
	move.l	d1,A1_FLAGS
	move.l	d1,A2_FLAGS
	
; Point A1BASE to the destination

	move.l	a1,A1_BASE

; Set the pixel pointer to the offset in d1

	move.l	#0,A1_PIXEL

; Find size of data to load

	asr.l	#1,d0			; Convert to words
	or.l	#$10000,d0		; Set 1 outer loop
	move.l	d0,B_COUNT

; Set up Counters register to number of words
; Point A2BASE to the source
; a0 now points to the data

	move.l	a0,A2_BASE

; Set the pixel pointer to the offset in d1

	move.l	#0,A2_PIXEL

.blit_go:
	move.l	#(SRCEN|LFU_AN|LFU_A),d0
	move.l	d0,B_CMD


; NOTE: No Wait for BLTTER Idle - I have yet to be overrun but WARNING

	movem.l	(sp)+,d0-d1/a0-a2	; restore registers
	rts

;*=============================================================================*
;* gpurun	- tell the GPU to begin execution			       *
;*									       *
;* Given:								       *
;*	a0 = 68000 address of GPU program				       *
;*	Control								       *
;*									       *
;* Returns:								       *
;*	Runs the GPU							       *
;*									       *
;* Register Usage:							       *
;*	All Registers Restored						       *
;*									       *
;* Externals:								       *
;*	None								       *
;*									       *
;*=============================================================================*

gpurun:
	movem.l	d0-d1/a0,-(sp)		; save GPU address for restore

	move.l	#$F03000,G_PC		; set GPU Program Counter
	move.l	#$11,G_CTRL		; Turn on the GPU

	movem.l	(sp)+,a0/d0-d1		; restore GPU address
	rts

;*=============================================================================*
;* GPUWAIT	- wait for the GPU to finish executing			       *
;*									       *
;* Given:								       *
;*	Control								       *
;*									       *
;* Returns:								       *
;*	waits for GPU to complete execution				       *
;*									       *
;* Register Usage:							       *
;*	All Registers Restored						       *
;*									       *
;* Externals:								       *
;*	None								       *
;*									       *
;*=============================================================================*

gpuwait:
	movem.l	a0-a1/d0-d1,-(sp)	; save registers

	lea	G_CTRL,a0
	move.l	#$400000,a1
.gpuwt:					; wait for GPU to finish
	move.l	framecnt,d1
	swap	d1
	move.w	$f00006,d1
	move.l	d1,(a1)+
	move.l	#-1,(a1)
	move.l	(a0),d0
	asr	#1,d0
	bcs	.gpuwt

	movem.l	(sp)+,a0-a1/d0-d1	; restore registers
	rts

;*******************************************************************************
;* END OF FILE								       *
;*******************************************************************************

	.end

